rm(list=ls())
library(foreign)
library(ggplot2)
library(stargazer)

sigma_theta<-sqrt(0.011+0.035) # sigma_psi^2, sigma_theta^2 from Schochett and Chiang, Table B-2, assuming sigma^2=1
sigma_eps<- sqrt(0.030 + 0.923) # sigma_omega^2, sigma_epsilon^2 from Schochett and Chiang, Table B-2
sigma_sq_theta<-sigma_theta^2
sigma_sq_eps<-sigma_eps^2

## LA Times data
data<-read.dta("../la_times_data/teachers_for_r.dta")
reg_Read_data<-lm(count ~ tcher4 + I(tcher4^2), data=data)
reg_Math_data<-lm(count ~ tchem4 + I(tchem4^2), data=data)

data_Read_coef<-coef(reg_Read_data)
data_Read_RSE<-sqrt(sum(reg_Read_data$residuals^2)/ reg_Read_data$df.residual)
data_Math_coef<-coef(reg_Math_data)
data_Math_RSE<-sqrt(sum(reg_Math_data$residuals^2)/ reg_Math_data$df.residual)
# note: 99.9th percentile class size is 40. minimum is 11. 
quantile(ecdf(data$count), .999)
quantile(ecdf(data$count), .001)

## 
avg_count<-mean(data$count)
sd_count<-sd(data$count)

## simulate theta
set.seed(9836)
n_tch_sim<-dim(data)[1]
theta_sim<-rnorm(n_tch_sim, mean=0, sd=sigma_theta)

## sample class sizes using draws from empirical distribution
n_sim<-sample(data$count, n_tch_sim, replace=T)

## simulate mean test score shocks, based on n_sim
eps_sim<-rnorm(n_tch_sim, mean=0, sd=sigma_eps/sqrt(n_sim))

## hat_theta
hat_theta_sim<-theta_sim + eps_sim

to_twist_n<-function(theta) -.1 + 0.15*theta - 4*theta^2
n_twisted_sim<-round(n_sim + to_twist_n(theta_sim), 0)

eps_twisted_sim<-rnorm(n_tch_sim, mean=0, sd=sigma_eps/sqrt(n_twisted_sim))
hat_theta_twisted_sim<-theta_sim + eps_twisted_sim

## baseline reg: can heteroskedasticity explain this?
reg_sim<-lm(n_sim ~ hat_theta_sim + I(hat_theta_sim^2))
baseline_coef<-coef(reg_sim)

q_vec<-1:3/4

targets_Read_data<-c(quantile(data$count, q_vec), data_Read_coef)
targets_Math_data<-c(quantile(data$count, q_vec), data_Math_coef)

more_targets_Read_data<-c(quantile(data$count, q_vec), data_Read_coef, data_Read_RSE)
more_targets_Math_data<-c(quantile(data$count, q_vec), data_Math_coef, data_Math_RSE)

reg_twisted_sim<-lm(n_twisted_sim ~ hat_theta_twisted_sim + I(hat_theta_twisted_sim^2))
twisted_coef<-coef(reg_twisted_sim)

## dumps above bits into function to calculate distance in coefficient

fit_n_theta<-function(targets_data) function(aa_vec)
{
	set.seed(9836)
	theta_sim<-rnorm(n_tch_sim, mean=0, sd=sigma_theta)

	n_sim<-sample(data$count, n_tch_sim, replace=T)

	to_twist_n<-function(theta) aa_vec[1] + aa_vec[2]*theta + aa_vec[3]*theta^2
	n_twisted_sim<-pmax(round(n_sim + to_twist_n(theta_sim), 0), 1)

	eps_twisted_sim<-rnorm(n_tch_sim, mean=0, sd=sigma_eps/sqrt(n_twisted_sim))
	hat_theta_twisted_sim<-theta_sim + eps_twisted_sim

	reg_twisted_sim<-lm(n_twisted_sim ~ hat_theta_twisted_sim + I(hat_theta_twisted_sim^2))
	twisted_coef<-coef(reg_twisted_sim)

	targets_twisted_sim<-c(quantile(n_twisted_sim, q_vec), twisted_coef)
	list(targets_twisted_sim=targets_twisted_sim, dist= sum((targets_twisted_sim-targets_data)^2))
}

## for Reading
for_optim_Read<-function(aa_vec) fit_n_theta(targets_Read_data)(aa_vec)$dist

indirect_inf_Read<-optim(c(-.5,.25,-10), for_optim_Read, method="Nelder-Mead", control=list(trace=5))
est_Read_aa_vec<-indirect_inf_Read$par

## for Math
for_optim_Math<-function(aa_vec) fit_n_theta(targets_Math_data)(aa_vec)$dist

indirect_inf_Math<-optim(c(-0.5,  .1, -0.1), for_optim_Math, lower=-20, upper=c(10,10,0), method="L-BFGS-B", control=list(trace=5))
est_Math_aa_vec<-indirect_inf_Math$par


## compare data and sim
quad_fit_Read_data<-function(x) sum(data_Read_coef * c(1,x,x^2))
quad_fit_Read_sim<-function(x) sum(est_Read_beta_vec * c(1,x,x^2))
quad_fit_Math_data<-function(x) sum(data_Math_coef * c(1,x,x^2))

quad_fit_Read_sim<-function(x) sum(est_Read_beta_vec * c(1,x,x^2))
quad_fit_Math_sim<-function(x) sum(est_Math_beta_vec * c(1,x,x^2))

count_plot_lim<-c(quantile(data$count, 0.15),quantile(data$count, 0.85))

Read_count_plot<-ggplot(data=data, aes(x=tcher4, y=count)) + geom_point(alpha=0.01) + geom_smooth(size=1) + xlim(-3*sd(data$tcher4), 3*sd(data$tcher4)) + xlab("Teacher quality, Reading") + ylab("Class size") + coord_cartesian(ylim=count_plot_lim) 

Math_count_plot<-ggplot(data=data, aes(x=tchem4, y=count)) + geom_point(alpha=0.01) + geom_smooth(size=1) + xlim(-3*sd(data$tchem4), 3*sd(data$tchem4)) + xlab("Teacher quality, Math") + ylab("Class size")+ coord_cartesian(ylim=count_plot_lim) 



################# more aux parameters
## include RSE in targets
more_fit_n_theta<-function(targets_data, weight_vec) function(par_vec)
{
	aa_vec<-par_vec[1:3]
	n_rand_sd<-max(par_vec[4], 1e-4)

	set.seed(9836)
	theta_sim<-rnorm(n_tch_sim, mean=0, sd=sigma_theta)

	n_sim<-rnorm(n_tch_sim, mean=0, sd=n_rand_sd)
	
	to_twist_n<-function(theta) aa_vec[1] + aa_vec[2]*theta + aa_vec[3]*theta^2
	n_twisted_sim<-pmax(round(n_sim + to_twist_n(theta_sim), 0), 1)

	eps_twisted_sim<-rnorm(n_tch_sim, mean=0, sd=sigma_eps/sqrt(n_twisted_sim))
	hat_theta_twisted_sim<-theta_sim + eps_twisted_sim

	reg_twisted_sim<-lm(n_twisted_sim ~ hat_theta_twisted_sim + I(hat_theta_twisted_sim^2))
	twisted_coef<-coef(reg_twisted_sim)
	twisted_RSE<-sqrt(sum(reg_twisted_sim$residuals^2)/reg_twisted_sim$df.residual)

	targets_twisted_sim<-c(quantile(n_twisted_sim, q_vec), twisted_coef, twisted_RSE)
	list(targets_twisted_sim=targets_twisted_sim, dist= sum(weight_vec*(targets_twisted_sim-targets_data)^2))
}

## for Reading
more_for_optim_Read<-function(par_vec) more_fit_n_theta(more_targets_Read_data,1)(par_vec)$dist
more_indirect_inf_Read<-optim(c(22,.25,-10, 3), more_for_optim_Read, method="Nelder-Mead", control=list(trace=5))
more_est_Read_aa_vec<-more_indirect_inf_Read$par[1:3]
more_est_Read_n_rand_sd<-more_indirect_inf_Read$par[4]


## for Math
more_for_optim_Math<-function(par_vec) more_fit_n_theta(more_targets_Math_data,c(rep(.25,3),2,1,.5,1))(par_vec)$dist

more_indirect_inf_Math<-optim(c(22,.05,-0.05, 3), more_for_optim_Math, method="Nelder-Mead", control=list(trace=5, parscale=c(1,0.1,.02,.1)))
more_est_Math_aa_vec<-more_indirect_inf_Math$par[1:3]
more_est_Math_n_rand_sd<-more_indirect_inf_Math$par[4]


stargazer(rbind(more_indirect_inf_Read$par, more_indirect_inf_Math$par))

