####
#Runs code for solving 
#	cutoff model
#	AS0
###

rm(list=ls())

library(reshape2)
library(ggplot2)
library(truncnorm)
library(xtable)
library(stargazer)
library(numDeriv)
library(foreign)
library(tikzDevice)

eps<-1e-6
der_bump<-1e-2
mu_theta<-0 # mean of teacher quality distribution
c_points<- -100:100/100 # grid for optimal cutoff policies
q_desired_vec<-seq(0.001,1-0.001, length=100) # desired cutoffs kappa

alpha_low<-.25
alpha_high<-2/3

###parameters for plotting
par_mgp<-c(2.5,1,0)
cex_lab_size<-1
one_pic_width<-4.5*(1.15^2)
one_pic_height<-one_pic_width
one_pic_pointsize<-12
two_pic_width<-4.5*1.25
two_pic_height<-two_pic_width
two_pic_pointsize<-12
three_pic_width<-4.5
three_pic_height<-three_pic_width
three_pic_pointsize<-12
###

sigma_theta<-sqrt(0.011+0.035) # sigma_psi^2, sigma_theta^2 from Schochett and Chiang, Table B-2, assuming sigma^2=1
sigma_eps<- sqrt(0.030 + 0.923) # sigma_omega^2, sigma_epsilon^2 from Schochett and Chiang, Table B-2

sigma_sq_theta<-sigma_theta^2
sigma_sq_eps<-sigma_eps^2

rho<-sigma_sq_theta / sigma_sq_eps

lambda<-function(estimator) function(n)
{
  1*(estimator=="FE") + 1*(estimator=="EB")*sigma_sq_theta/(sigma_sq_theta + sigma_sq_eps/n)
}

sigma_hat_theta_sq<-function(estimator) function(n)
{
  lambda(estimator)(n)^2*(sigma_sq_theta + sigma_sq_eps/n)
}

d_lambda_d_n<-function(n)
{
  rho^-1/n^2 * (1/(1+rho^-1/n))^2
}

############################
#Model

# benefit to correctly assigning bonus to good teacher, FE and RE
reward_good<-function(estimator) function(c_desired) function(c_endog) 
{
  integrate(function(theta) {dtruncnorm(theta, a=c_desired, b=Inf, mean=0,sd=sigma_theta) * (1 - pnorm(c_endog/lambda(estimator)(n(theta)) -theta, mean=0, sd=sqrt(sigma_sq_eps/n(theta))))}, lower=c_desired, upper=Inf)$value
}

# benefit to correctly not assigning bonus to bad teacher, FE and RE
reward_bad<-function(estimator) function(c_desired) function(c_endog) 
{
  integrate(function(theta) {dtruncnorm(theta, a=-Inf, b=c_desired, mean=0,sd=sigma_theta) * (pnorm(c_endog/lambda(estimator)(n(theta))-theta, mean=0, sd=sqrt(sigma_sq_eps/n(theta))))}, lower=-Inf, upper=c_desired)$value
}

reward_total<-function(estimator) function(c_desired) function(alpha) function(c_endog)
{
  alpha*reward_good(estimator)(c_desired)(c_endog) + (1-alpha)*reward_bad(estimator)(c_desired)(c_endog)
}


solve_for_c<-function(estimator) function(c_desired) function(alpha)
{
  optimize(function(c_endog) -Vectorize(reward_total(estimator)(c_desired)(alpha))(c_endog), lower=min(c_points), upper=max(c_points))
}

FOC_obj_wrt_c_given_theta_below<-function(estimator) function(c_endog) function(theta)
{
  1/(lambda(estimator)(n(theta)) * sqrt(sigma_sq_eps/n(theta))) * 
    dnorm(c_endog/lambda(estimator)(n(theta)) -theta, mean=0, sd=sqrt(sigma_sq_eps/n(theta)))
}

FOC_obj_wrt_c_given_theta_above<-function(estimator) function(c_endog) function(theta)
{
  1/(lambda(estimator)(n(theta)) * sqrt(sigma_sq_eps/n(theta))) * 
    dnorm(c_endog/lambda(estimator)(n(theta)) -theta, mean=0, sd=sqrt(sigma_sq_eps/n(theta)))
}

FOC_obj_wrt_c<-function(estimator) function(c_desired) function(alpha) function(c_endog) 
{
 (1-alpha)* integrate(function(x) FOC_obj_wrt_c_given_theta_below(estimator)(c_endog)(x)*dtruncnorm(x, a=-Inf, b=c_desired, mean = 0, sd=sigma_theta), lower=-Inf, upper=c_desired)$value -
(alpha)* integrate(function(x) FOC_obj_wrt_c_given_theta_above(estimator)(c_endog)(x)*dtruncnorm(x, a=c_desired, b=Inf, mean = 0, sd=sigma_theta), lower=c_desired, upper=Inf)$value
  }

solve_using_FOC_obj_wrt_c<-function(estimator) function(c_desired) function(alpha) 
{
  optimize(function(c_endog) (Vectorize(FOC_obj_wrt_c(estimator)(c_desired)(alpha))(c_endog))^2, lower=min(c_points), upper=max(c_points))
}

## should always be negative
d_FOC_d_c<-function(estimator) function(c_desired) function(alpha) 
{
  c_endog_in<-solve_for_c(estimator)(c_desired)(alpha)$minimum
  
  (FOC_obj_wrt_c(estimator)(c_desired)(alpha)(c_endog_in+der_bump)-FOC_obj_wrt_c(estimator)(c_desired)(alpha)(c_endog_in-der_bump))/(2*der_bump)
}
  
d_FOC_d_kappa<-function(estimator) function(c_desired) function(alpha) 
{
  c_endog_in<-solve_for_c(estimator)(c_desired)(alpha)$minimum
  
  (FOC_obj_wrt_c(estimator)(c_desired+der_bump)(alpha)(c_endog_in)-FOC_obj_wrt_c(estimator)(c_desired-der_bump)(alpha)(c_endog_in))/(2*der_bump)
}

## to check d_FOC_d_kappa analytical
d_FOC_d_kappa_analytical<-function(estimator) function(c_desired) function(alpha) 
{
  stopifnot(c_desired==0 & alpha==0.5)
  stopifnot(n(-.25)==n(.25))
  
  2*dnorm(0,mean=0, sd=sigma_theta)*1/(lambda(estimator)(n(c_desired)) * sqrt(sigma_sq_eps/n(c_desired)))*dnorm(0,mean=0, sd=sqrt(sigma_sq_eps/n(c_desired)))
}

## to check d_FOC_d_c analytical
d_FOC_d_c_analytical<-function(estimator) function(c_desired) function(alpha) 
{
  stopifnot(c_desired==0 & alpha==0.5)
  stopifnot(n(-.25)==n(.25))
  
  g_tmp<-function(theta_in) 
    {
      1/(lambda(estimator)(n(theta_in))^2 * sqrt(sigma_sq_eps/n(theta_in))^3)*dnorm(theta_in/sqrt(sigma_sq_eps/n(theta_in))) * theta_in * dnorm(theta_in, mean=0, sd=sigma_theta)
  }
  integrate(g_tmp, lower=-Inf, upper=c_desired)$value - integrate(g_tmp, lower=c_desired, upper=Inf)$value
  }

## uses IFT 
d_c_d_kappa<-function(estimator) function(c_desired) function(alpha) 
{
  -d_FOC_d_kappa(estimator)(c_desired)(alpha) / d_FOC_d_c(estimator)(c_desired)(alpha)
}

d_c_d_kappa_analytical<-function(estimator) function(c_desired) function(alpha) 
{
  -d_FOC_d_kappa_analytical(estimator)(c_desired)(alpha) / d_FOC_d_c_analytical(estimator)(c_desired)(alpha)
}

## probability of being above cutoff, given true theta
prob_above_c_endog_given_theta<-function(estimator) function(c_desired) function(c_endog) function(theta)
{
	(1 - pnorm(c_endog/lambda(estimator)(n(theta)) -theta, mean=0, sd=sqrt(sigma_sq_eps/n(theta))))
}

prob_above_c_endog_star_given_theta<-function(estimator) function(c_desired) function(alpha) function(theta)
{
c_endog<-solve_for_c(estimator)(c_desired)(alpha)$minimum
	(1 - pnorm(c_endog/lambda(estimator)(n(theta)) -theta, mean=0, sd=sqrt(sigma_sq_eps/n(theta))))
}


# takes in cutoff percentile vec desired_cut_vec
# solves for v^FE(desired), v^EB(desired)
calc_v_vec<-function(q_desired_vec) function(alpha) function(scenario)
{
  c_desired<-qnorm(q_desired_vec, mean=0, sd=sigma_theta)
  
  c_soln_FE_df<-data.frame(c_endog=numeric(0), obj=numeric(0))
  c_soln_EB_df<-data.frame(c_endog=numeric(0), obj=numeric(0))
  
  for (c_desired_tmp in c_desired)
  {
    c_soln_FE_df<-rbind(c_soln_FE_df, solve_for_c("FE")(c_desired_tmp)(alpha))
    c_soln_EB_df<-rbind(c_soln_EB_df, solve_for_c("EB")(c_desired_tmp)(alpha))
  }
  df<-cbind(c_soln_FE_df, c_soln_EB_df)
  names(df)<-c("FE_c_endog", "FE_obj", "EB_c_endog", "EB_obj")
  
  # reverse because optim minimizes -ve objective
  df$FE_obj<- -df$FE_obj
  df$EB_obj<- -df$EB_obj
  
  df$FE_minus_EB <-df$FE_obj - df$EB_obj
  df$FE_over_EB  <-df$FE_obj/df$EB_obj
  df$q_desired<-q_desired_vec
  df$kappa_desired<-c_desired
  df
}

plot_stuff<-function(estimator) function(c_desired) function(alpha)
{
  c_soln<-solve_for_c(estimator)(c_desired)(alpha)
  
  reward_points<-Vectorize(reward_total(estimator)(c_desired)(alpha))(c_points)
    
  plot(c_points, reward_points, t="l", col="black", ylim=range(c(0, reward_points)))
  points(c_points, (1-alpha)*Vectorize(reward_bad(estimator)(c_desired))(c_points), t="l", col="blue")
  points(c_points, alpha*Vectorize(reward_good(estimator)(c_desired))(c_points), t="l", col="red")
  abline(v=c_soln$minimum)
  print(c_soln)
}

plot_both_cutoffs<-function(df_in)
{
  attach(df_in)
  plot(kappa_desired, FE_c_endog, ylim=range(c(FE_c_endog,EB_c_endog, kappa_desired)), t="l", col="blue")
  points(kappa_desired, EB_c_endog, t="l", col="red")
  abline(a=0,b=1,lty=3)
  legend("topleft", legend=c("FE", "EB"), col = c("blue", "red"), lty=1, bty="n")
  detach(df_in)
}

frac_below_c_endog<-function(estimator) function(c_endog)
{
    integrate(function(theta) {dnorm(theta, mean=0,sd=sigma_theta) * (pnorm(c_endog/lambda(estimator)(n(theta))-theta, mean=0, sd=sqrt(sigma_sq_eps/n(theta))))}, lower=-Inf, upper=Inf)$value
}

pretty_rewards<-function(estimator) function(c_desired) function(c_endog) function(total_reward)
{
  data.frame(c_endog=c_endog,     reward_good=reward_good(estimator)(c_desired)(c_endog), reward_bad=reward_bad(estimator)(c_desired)(c_endog), total_reward=total_reward, frac_below_cut=frac_below_c_endog(estimator)(c_endog)) 
}

# takes in desired cutoff percentile (e.g., 0.01 or 0.99)
# Fig 1a-c
compare_stuff<-function(q_desired) function(alpha) function(scenario)
{
  c_desired<-qnorm(q_desired, mean=0, sd=sigma_theta)
  
  c_soln_FE<- solve_for_c("FE")(c_desired)(alpha)
  c_soln_EB<- solve_for_c("EB")(c_desired)(alpha)
    
  reward_points_FE<-Vectorize(reward_total("FE")(c_desired)(alpha))(c_points)
  reward_points_EB<-Vectorize(reward_total("EB")(c_desired)(alpha))(c_points)
    
  par(xpd=T, mar=par()$mar+c(0,0,0,6))
  
  pdf(paste("./pdfs/", scenario, "_", q_desired*100, ".pdf", sep=""))
  plot(c_points, reward_points_FE, t="l", col="red", ylim=range(c(reward_points_FE, reward_points_EB)), xlab=expression("Cutoff policy" ~ "c"), ylab=expression("Expected utility" ~ E[theta ~bar(epsilon)](theta, hat(theta)~ ";"~ "c", ~ kappa) ), lty=1, cex.lab=cex_lab_size)
  points(c_points, reward_points_EB, t="l", col="blue", lty=2)
  abline(v=c_soln_FE$minimum, lty=1, col="red")
  abline(v=c_soln_EB$minimum, lty=2, col="blue")
  text(c_soln_FE$minimum,min(c(reward_points_FE, reward_points_EB)), expression(c^{"*FE"}), cex=0.75, adj=c(0,0))
  text(c_soln_EB$minimum,min(c(reward_points_FE, reward_points_EB)), expression(c^{"*EB"}), cex=0.75, adj=c(0,1))
    
  legend("topright",c("FE", "EB"), lty = c(1,2), col=c("red", "blue"), bty="n")
  dev.off()

  tikz(paste("./pdfs/", scenario, "_", q_desired*100, ".tex", sep=""), standAlone=F, width=three_pic_width, height=three_pic_height, pointsize=three_pic_pointsize)
  plot(c_points, reward_points_FE, t="l", col="red", ylim=range(c(reward_points_FE, reward_points_EB)), xlab="Cutoff policy $c$", ylab=
"Expected utility $\\textrm{E}[u(\\theta,\\hat{\\theta};c,\\kappa)]$", lty=1, cex.lab=cex_lab_size)
  points(c_points, reward_points_EB, t="l", col="blue", lty=2)
  abline(v=c_soln_FE$minimum, lty=1, col="red")
  abline(v=c_soln_EB$minimum, lty=2, col="blue")
  text(c_soln_FE$minimum,min(c(reward_points_FE, reward_points_EB)), "$c^{*FE}$", cex=0.75, adj=c(0,0))
  text(c_soln_EB$minimum,min(c(reward_points_FE, reward_points_EB)), "$c^{*EB}$", cex=0.75, adj=c(0,1))
    
  legend("topright",c("FE", "EB"), lty = c(1,2), col=c("red", "blue"), bty="n")
  dev.off()

  par(mar=c(5, 4, 4, 2) + 0.1)
  
  results<-rbind("FE"= pretty_rewards("FE")(c_desired)(c_soln_FE$minimum)(-c_soln_FE$objective), 
                 "EB"= pretty_rewards("EB")(c_desired)(c_soln_EB$minimum)(-c_soln_EB$objective))
  results<-cbind(results, "comp_with_FE"=c(0, -c_soln_FE$objective + c_soln_EB$objective))
  results<-cbind(results, "scenario"=scenario)
  results<-cbind(results, "cut desired"=q_desired)
  results
}

######################### RUN SIMULATION
  # class size increasing in theta
  n<-function(theta) punif(theta, min=qnorm(0.01, mean=mu_theta, sd=sigma_theta), max=qnorm(0.99, mean=mu_theta, sd=sigma_theta))*15+15
  
  r8<-compare_stuff(0.01)(0.5)("increasing") 
  r9<-compare_stuff(0.05)(0.5)("increasing")
  r10<-compare_stuff(0.10)(0.5)("increasing")
  r11<-compare_stuff(0.15)(0.5)("increasing")
  r12<-compare_stuff(0.50)(0.5)("increasing")
  r13<-compare_stuff(0.85)(0.5)("increasing")
  r14<-compare_stuff(0.95)(0.5)("increasing")
  
  # constant class size
  calc_avg_class_size<-function() integrate(function(x) n(x)*dnorm(x, mean=mu_theta, sd=sigma_theta), lower=-Inf, upper=Inf)$value
  avg_class_size<-calc_avg_class_size()
  
  n<-function(theta) {avg_class_size +theta*0}
  
  r1<-compare_stuff(0.01)(0.5)("constant25")
  r2<-compare_stuff(0.05)(0.5)("constant25")
  r3<-compare_stuff(0.10)(0.5)("constant25")
  r4<-compare_stuff(0.15)(0.5)("constant25")
  r5<-compare_stuff(0.50)(0.5)("constant25")
  r6<-compare_stuff(0.85)(0.5)("constant25")
  r7<-compare_stuff(0.95)(0.5)("constant25")
  
### Constant n
plots_madeup_ylim<-c(.985,1.02)
plots_madeup_ylim_alpha_extreme<-c(.995,1.005)
n<-function(theta) 25 + theta*0

df_obj_constant<-calc_v_vec(q_desired_vec)(0.5)(" ")
df_obj_constant_alpha_low<-calc_v_vec(q_desired_vec)(alpha_low)(" ")
df_obj_constant_alpha_high<-calc_v_vec(q_desired_vec)(alpha_high)(" ")
df_obj_constant_alpha_very_high<-calc_v_vec(q_desired_vec)(alpha_very_high)(" ")
df_obj_constant_alpha_very_low<-calc_v_vec(q_desired_vec)(alpha_very_low)(" ")
pdf("./pdfs/df_obj_constant.pdf")
par(mgp=par_mgp)
  plot(q_desired_vec, df_obj_constant$FE_over_EB, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab=expression(v(kappa)^{"FE"} / v(kappa)^{"EB"}), cex.lab=cex_lab_size, ylim=plots_madeup_ylim)
  abline(h=1, lty=3)
dev.off()

### Increasing n
smallest_class_for_madeup_n<-10
biggest_class_for_madeup_n<-40

n_nonvec<-function(theta) min(biggest_class_for_madeup_n, max(smallest_class_for_madeup_n, 25+10*theta)) 
n<-function(theta) Vectorize(n_nonvec)(theta)
calc_avg_class_size()

df_obj_increasing<-calc_v_vec(q_desired_vec)(0.5)(" ")
df_obj_increasing_alpha_low<-calc_v_vec(q_desired_vec)(alpha_low)(" ")
df_obj_increasing_alpha_high<-calc_v_vec(q_desired_vec)(alpha_high)(" ")
df_obj_increasing_alpha_very_high<-calc_v_vec(q_desired_vec)(alpha_very_high)(" ")
df_obj_increasing_alpha_very_low<-calc_v_vec(q_desired_vec)(alpha_very_low)(" ")

pdf("./pdfs/df_obj_increasing.pdf")
par(mgp=par_mgp)
plot(q_desired_vec, df_obj_increasing$FE_over_EB, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab=expression(v(kappa)^{"FE"} / v(kappa)^{"EB"}), cex.lab=cex_lab_size, ylim=plots_madeup_ylim)
abline(h=1, lty=3)
dev.off()

### Example negative quadratic n
n_nonvec<-function(theta) min(biggest_class_for_madeup_n, max(smallest_class_for_madeup_n, 25.46 - 10*theta^2)) 
n<-function(theta) Vectorize(n_nonvec)(theta)
calc_avg_class_size()

  r15<-compare_stuff(0.01)(0.5)("neg_quad") 
  r16<-compare_stuff(0.05)(0.5)("neg_quad")
  r17<-compare_stuff(0.10)(0.5)("neg_quad")
  r18<-compare_stuff(0.15)(0.5)("neg_quad")
  r19<-compare_stuff(0.50)(0.5)("neg_quad")
  r20<-compare_stuff(0.85)(0.5)("neg_quad")
  r21<-compare_stuff(0.95)(0.5)("neg_quad")

  all_together<-rbind(r1, r2)
  all_together<-rbind(all_together, r3)
  all_together<-rbind(all_together, r4)
  all_together<-rbind(all_together, r5)
  all_together<-rbind(all_together, r6)
  all_together<-rbind(all_together, r7)
  all_together<-rbind(all_together, r8)
  all_together<-rbind(all_together, r9)
  all_together<-rbind(all_together, r10)
  all_together<-rbind(all_together, r11)
  all_together<-rbind(all_together, r12)
  all_together<-rbind(all_together, r13)
  all_together<-rbind(all_together, r14)
  all_together<-rbind(all_together, r15)
  all_together<-rbind(all_together, r16)
  all_together<-rbind(all_together, r17)
  all_together<-rbind(all_together, r18)
  all_together<-rbind(all_together, r19)
  all_together<-rbind(all_together, r20)
  all_together<-rbind(all_together, r21)

  
  write.csv(all_together, file="./pdfs/all_together.csv")

df_obj_negative_quadratic<-calc_v_vec(q_desired_vec)(0.5)(" ")
df_obj_negative_quadratic_alpha_low<-calc_v_vec(q_desired_vec)(alpha_low)(" ")
df_obj_negative_quadratic_alpha_high<-calc_v_vec(q_desired_vec)(alpha_high)(" ")
df_obj_negative_quadratic_alpha_very_high<-calc_v_vec(q_desired_vec)(alpha_very_high)(" ")
df_obj_negative_quadratic_alpha_very_low<-calc_v_vec(q_desired_vec)(alpha_very_low)(" ")

### for calculating d c/d alpha
df_obj_negative_quadratic_bump<-calc_v_vec(q_desired_vec)(0.5+0.01)(" ")
df_obj_negative_quadratic_alpha_low_bump<-calc_v_vec(q_desired_vec)(alpha_low+0.01)(" ")
df_obj_negative_quadratic_alpha_high_bump<-calc_v_vec(q_desired_vec)(alpha_high+0.01)(" ")
df_obj_negative_quadratic_alpha_very_high_bump<-calc_v_vec(q_desired_vec)(alpha_very_high+0.01)(" ")
df_obj_negative_quadratic_alpha_very_low_bump<-calc_v_vec(q_desired_vec)(alpha_very_low+0.01)(" ")

summary((df_obj_negative_quadratic_bump$EB_c_endog-df_obj_negative_quadratic$EB_c_endog)/0.01)
summary((df_obj_negative_quadratic_alpha_low_bump$EB_c_endog-df_obj_negative_quadratic_alpha_low$EB_c_endog)/0.01)
summary((df_obj_negative_quadratic_alpha_high_bump$EB_c_endog-df_obj_negative_quadratic_alpha_high$EB_c_endog)/0.01)
summary((df_obj_negative_quadratic_alpha_very_low_bump$EB_c_endog-df_obj_negative_quadratic_alpha_very_low$EB_c_endog)/0.01)
summary((df_obj_negative_quadratic_alpha_very_high_bump$EB_c_endog-df_obj_negative_quadratic_alpha_very_high$EB_c_endog)/0.01)


pdf("./pdfs/df_obj_negative_quadratic.pdf")
par(mgp=par_mgp)
plot(q_desired_vec, df_obj_negative_quadratic$FE_over_EB, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab=expression(v(kappa)^{"FE"} / v(kappa)^{"EB"}), cex.lab=cex_lab_size, ylim=plots_madeup_ylim)
abline(h=1, lty=3)
dev.off()

### Made up positive quadratic n
n_nonvec<-function(theta) min(biggest_class_for_madeup_n, max(smallest_class_for_madeup_n, 24.54 + 10*theta^2)) 
n<-function(theta) Vectorize(n_nonvec)(theta)
calc_avg_class_size()

df_obj_positive_quadratic<-calc_v_vec(q_desired_vec)(0.5)(" ")
df_obj_positive_quadratic_alpha_low<-calc_v_vec(q_desired_vec)(alpha_low)(" ")
df_obj_positive_quadratic_alpha_high<-calc_v_vec(q_desired_vec)(alpha_high)(" ")
df_obj_positive_quadratic_alpha_very_high<-calc_v_vec(q_desired_vec)(alpha_very_high)(" ")
df_obj_positive_quadratic_alpha_very_low<-calc_v_vec(q_desired_vec)(alpha_very_low)(" ")

pdf("./pdfs/df_obj_positive_quadratic.pdf")
par(mgp=par_mgp)
plot(q_desired_vec, df_obj_positive_quadratic$FE_over_EB, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab=expression(v(kappa)^{"FE"} / v(kappa)^{"EB"}), cex.lab=cex_lab_size, ylim=plots_madeup_ylim)
abline(h=1, lty=3)
dev.off()

pdf("./pdfs/df_obj_all.pdf")
par(mgp=par_mgp)
  plot(q_desired_vec, df_obj_constant$FE_over_EB, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab=expression("Ratio of administrator value" ~ v(kappa)^{"FE"} / v(kappa)^{"EB"}), cex.lab=cex_lab_size, axes=F, ylim=plots_madeup_ylim, lty=3)

  Axis(side=1, labels=T)
  axis(side=2, at=1, tick=F)
  Axis(side=2, labels=F)

  points(q_desired_vec, df_obj_increasing$FE_over_EB, t="l", lty=2, col="red")
  points(q_desired_vec, df_obj_negative_quadratic$FE_over_EB, t="l", lty=4, col="blue")
  points(q_desired_vec, df_obj_positive_quadratic$FE_over_EB, t="l", lty=5, col="brown")
  
  legend(0.55,plots_madeup_ylim[2]+0.001,c("constant", "increasing", "negative quadratic", "positive quadratic"), lty = c(3,2,4,5), col=c("black","red", "blue", "brown"), bty="n")

dev.off()

tikz("./pdfs/df_obj_all.tex", standAlone=F, width=one_pic_width, height=one_pic_height, pointsize=one_pic_pointsize)
par(mgp=par_mgp)
  plot(q_desired_vec, df_obj_constant$FE_over_EB, t="l", xlab="Desired cut percentile $F(\\kappa)$", ylab="Ratio of administrator value $v_{CP}^{FE}(\\kappa)/v_{CP}^{EB}(\\kappa)$", cex.lab=cex_lab_size, axes=F, ylim=plots_madeup_ylim, lty=3)

  Axis(side=1, labels=T)
  axis(side=2, at=1, tick=F)
  Axis(side=2, labels=F)

  points(q_desired_vec, df_obj_increasing$FE_over_EB, t="l", lty=2, col="red")
  points(q_desired_vec, df_obj_negative_quadratic$FE_over_EB, t="l", lty=4, col="blue")
  points(q_desired_vec, df_obj_positive_quadratic$FE_over_EB, t="l", lty=5, col="brown")
  
  legend(0.70,plots_madeup_ylim[2]+0.001,c("const.", "inc.", "neg. quad.", "pos. quad."), lty = c(3,2,4,5), col=c("black","red", "blue", "brown"), bty="n")

dev.off()


plots_madeup_ylim_alpha_low<-plots_madeup_ylim

## alpha is low
## Fig 8a
pdf("./pdfs/df_obj_all_alpha_low.pdf")
par(mgp=par_mgp)

plot(q_desired_vec, df_obj_constant_alpha_low$FE_over_EB, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab=expression("Ratio of administrator value" ~ v(kappa)^{"FE"} / v(kappa)^{"EB"}), cex.lab=cex_lab_size, axes=F, ylim=plots_madeup_ylim_alpha_low, lty=3)

Axis(side=1, labels=T)
axis(side=2, at=1, tick=F)
Axis(side=2, labels=F)

points(q_desired_vec, df_obj_increasing_alpha_low$FE_over_EB, t="l", lty=2, col="red")
points(q_desired_vec, df_obj_negative_quadratic_alpha_low$FE_over_EB, t="l", lty=4, col="blue")
points(q_desired_vec, df_obj_positive_quadratic_alpha_low$FE_over_EB, t="l", lty=5, col="brown")

legend(0.55,plots_madeup_ylim_alpha_low[2]+0.0005,c("constant", "increasing", "negative quadratic", "positive quadratic"), lty = c(3,2,4,5), col=c("black","red", "blue", "brown"), bty="n")

dev.off()

tikz("./pdfs/df_obj_all_alpha_low.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
par(mgp=par_mgp)
  plot(q_desired_vec, df_obj_constant_alpha_low$FE_over_EB, t="l", xlab="Desired cut percentile $F(\\kappa)$", ylab="Ratio of administrator value $v_{CP}^{FE}(\\kappa)/v_{CP}^{EB}(\\kappa)$", cex.lab=cex_lab_size, axes=F, ylim=plots_madeup_ylim, lty=3)

  Axis(side=1, labels=T)
  axis(side=2, at=1, tick=F)
  Axis(side=2, labels=F)

  points(q_desired_vec, df_obj_increasing_alpha_low$FE_over_EB, t="l", lty=2, col="red")
  points(q_desired_vec, df_obj_negative_quadratic_alpha_low$FE_over_EB, t="l", lty=4, col="blue")
  points(q_desired_vec, df_obj_positive_quadratic_alpha_low$FE_over_EB, t="l", lty=5, col="brown")
  
  legend(0.70,plots_madeup_ylim_alpha_low[2]+0.002,c("const.", "inc.", "neg. quad.", "pos. quad."), lty = c(3,2,4,5), col=c("black","red", "blue", "brown"), bty="n")

dev.off()

## alpha is high
## Fig 8b
plots_madeup_ylim_alpha_high<-plots_madeup_ylim_alpha_low
pdf("./pdfs/df_obj_all_alpha_high.pdf")
par(mgp=par_mgp)
plot(q_desired_vec, df_obj_constant_alpha_high$FE_over_EB, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab=expression("Ratio of administrator value" ~ v(kappa)^{"FE"} / v(kappa)^{"EB"}), cex.lab=cex_lab_size, axes=F, ylim=plots_madeup_ylim_alpha_high, lty=3)

Axis(side=1, labels=T)
axis(side=2, at=1, tick=F)
Axis(side=2, labels=F)

points(q_desired_vec, df_obj_increasing_alpha_high$FE_over_EB, t="l", lty=2, col="red")
points(q_desired_vec, df_obj_negative_quadratic_alpha_high$FE_over_EB, t="l", lty=4, col="blue")
points(q_desired_vec, df_obj_positive_quadratic_alpha_high$FE_over_EB, t="l", lty=5, col="brown")

legend(0.55,plots_madeup_ylim_alpha_high[2]+0.0005,c("constant", "increasing", "negative quadratic", "positive quadratic"), lty = c(3,2,4,5), col=c("black","red", "blue", "brown"), bty="n")

dev.off()

tikz("./pdfs/df_obj_all_alpha_high.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
par(mgp=par_mgp)
  plot(q_desired_vec, df_obj_constant_alpha_high$FE_over_EB, t="l", xlab="Desired cut percentile $F(\\kappa)$", ylab="Ratio of administrator value $v_{CP}^{FE}(\\kappa)/v_{CP}^{EB}(\\kappa)$", cex.lab=cex_lab_size, axes=F, ylim=plots_madeup_ylim, lty=3)

  Axis(side=1, labels=T)
  axis(side=2, at=1, tick=F)
  Axis(side=2, labels=F)

  points(q_desired_vec, df_obj_increasing_alpha_high$FE_over_EB, t="l", lty=2, col="red")
  points(q_desired_vec, df_obj_negative_quadratic_alpha_high$FE_over_EB, t="l", lty=4, col="blue")
  points(q_desired_vec, df_obj_positive_quadratic_alpha_high$FE_over_EB, t="l", lty=5, col="brown")
  
  legend(0.70,plots_madeup_ylim_alpha_high[2]+0.002,c("const.", "inc.", "neg. quad.", "pos. quad."), lty = c(3,2,4,5), col=c("black","red", "blue", "brown"), bty="n")

dev.off()

### LA Times reading n
n_nonvec<-function(theta) max(11, 22.702 + 1.031 * theta + -13.929 *theta^2) # 11 is smallest class size in RAND study

n<-function(theta) Vectorize(n_nonvec)(theta)
 
### prob of being above c_endog_star
## Fig 5a-b
## desired cutoff in 1st percentile
theta_grid<-seq(-4*sigma_theta, 2*sigma_theta, length=100)

c_desired_tmp<-qnorm(.01, mean=0, sd=sigma_theta)
pdf("./pdfs/prob_above_cutoff_1.pdf")
	par(mgp=par_mgp)
	plot(theta_grid, prob_above_c_endog_star_given_theta(estimator="FE")(c_desired=c_desired_tmp)(alpha=0.5)(theta=theta_grid), t="l", lty=1, col="red", xlab="Teacher quality", ylab="Prob. above cutoff policy", ylim=c(0,1))
	points(theta_grid, prob_above_c_endog_star_given_theta(estimator="EB")(c_desired=c_desired_tmp)(alpha=0.5)(theta=theta_grid), t="l", lty=2, col="blue")
	abline(v=c_desired_tmp, lty=3)
	rect(min(theta_grid)-0.01, -.1, c_desired_tmp, 1.5, col= rgb(0,0,1.0,alpha=0.15), border=F)
	legend("bottomright",c(expression(kappa), expression(theta < kappa), "FE", "EB"), lty = c(3,0,1,2), pch=c(NA,22,NA,NA), pt.cex=3,pt.bg=c("black",rgb(0,0,1.0,alpha=0.15), "red", "blue"), col=c("black",rgb(0,0,1.0,alpha=0.15), "red", "blue"), bty="n", border=T)
dev.off()

tikz("./pdfs/prob_above_cutoff_1.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
	par(mgp=par_mgp)
	plot(theta_grid, prob_above_c_endog_star_given_theta(estimator="FE")(c_desired=c_desired_tmp)(alpha=0.5)(theta=theta_grid), t="l", lty=1, col="red", xlab="Teacher quality $\\theta$", ylab="Prob. above cutoff policy $\\textrm{Pr}\\{\\hat{\\theta}\\geq c^{*}\\}$", ylim=c(0,1))
	points(theta_grid, prob_above_c_endog_star_given_theta(estimator="EB")(c_desired=c_desired_tmp)(alpha=0.5)(theta=theta_grid), t="l", lty=2, col="blue")
	abline(v=c_desired_tmp, lty=3)
	rect(min(theta_grid)-0.01, -.1, c_desired_tmp, 1.5, col= rgb(0,0,1.0,alpha=0.15), border=F)
	legend(.18,.20,c("$\\kappa$", "$\\theta < \\kappa$", "FE", "EB"), lty = c(3,0,1,2), pch=c(NA,22,NA,NA), pt.cex=3,pt.bg=c("black", rgb(0,0,1.0,alpha=0.15), "red", "blue"), col=c("black", rgb(0,0,1.0,alpha=0.15), "red", "blue"), bty="n", border=T)
dev.off()

pdf("./pdfs/prob_ratio_above_cutoff_1.pdf")
	par(mgp=par_mgp)
	plot(theta_grid, prob_above_c_endog_star_given_theta(estimator="FE")(c_desired=c_desired_tmp)(alpha=0.5)(theta=theta_grid)/prob_above_c_endog_star_given_theta(estimator="EB")(c_desired=c_desired_tmp)(alpha=0.5)(theta=theta_grid), t="l", lty=1, col="black", xlab="Teacher quality", ylab="Ratio of prob. above cutoff policy (FE/EB)")
	abline(v=c_desired_tmp, lty=3)
	abline(h=1, lty=2)
	rect(min(theta_grid)-0.01, -.1, c_desired_tmp, 1.5, col= rgb(0,0,1.0,alpha=0.15), border=F)
	legend("bottomright",c(expression(kappa), expression(theta < kappa)), lty = c(3,0), pch=c(NA,22), pt.cex=3,pt.bg=c("black",rgb(0,0,1.0,alpha=0.15)), col=c("black",rgb(0,0,1.0,alpha=0.15)), bty="n", border=T)
dev.off()

tikz("./pdfs/prob_ratio_above_cutoff_1.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
	par(mgp=par_mgp)
	plot(theta_grid, prob_above_c_endog_star_given_theta(estimator="FE")(c_desired=c_desired_tmp)(alpha=0.5)(theta=theta_grid)/prob_above_c_endog_star_given_theta(estimator="EB")(c_desired=c_desired_tmp)(alpha=0.5)(theta=theta_grid), t="l", lty=1, col="black", xlab="Teacher quality $\\theta$", ylab="Ratio of prob. above cutoff policy $\\textrm{Pr}\\{\\hat{\\theta}\\geq c^{*FE}\\}/\\textrm{Pr}\\{\\hat{\\theta}\\geq c^{*EB}\\}$")
	abline(v=c_desired_tmp, lty=3)
	abline(h=1, lty=2)
	rect(min(theta_grid)-0.01, -.1, c_desired_tmp, 1.5, col= rgb(0,0,1.0,alpha=0.15), border=F)
	legend(.18,.60,c("$\\kappa$", "$\\theta < \\kappa$"), lty = c(3,0), pch=c(NA,22), pt.cex=3,pt.bg=c("black", rgb(0,0,1.0,alpha=0.15)), col=c("black", rgb(0,0,1.0,alpha=0.15)), bty="n", border=T)
dev.off()


pdf("./pdfs/plot_cutoffs.pdf")
par(mfrow=c(3,2))
attach(df_obj_constant)
plot(kappa_desired, FE_c_endog, t="l", col="red", main="constant")
points(kappa_desired, EB_c_endog, t="l", col="blue")
abline(a=0,b=1, lty=3)
detach(df_obj_constant)

attach(df_obj_increasing)
plot(kappa_desired, FE_c_endog, t="l", col="red", main="inc.")
points(kappa_desired, EB_c_endog, t="l", col="blue")
abline(a=0,b=1, lty=3)
detach(df_obj_increasing)

attach(df_obj_increasing_alpha_high)
plot(kappa_desired, FE_c_endog, t="l", col="red", main="inc., alpha high")
points(kappa_desired, EB_c_endog, t="l", col="blue")
abline(a=0,b=1, lty=3)
detach(df_obj_increasing_alpha_high)

attach(df_obj_negative_quadratic)
plot(kappa_desired, FE_c_endog, t="l", col="red", main="neg quad")
points(kappa_desired, EB_c_endog, t="l", col="blue")
abline(a=0,b=1, lty=3)
detach(df_obj_negative_quadratic)

attach(df_obj_negative_quadratic_alpha_high)
plot(kappa_desired, FE_c_endog, t="l", col="red", main="neg quad, alpha high")
points(kappa_desired, EB_c_endog, t="l", col="blue")
abline(a=0,b=1, lty=3)
detach(df_obj_negative_quadratic_alpha_high)

attach(df_obj_negative_quadratic_alpha_low)
plot(kappa_desired, FE_c_endog, t="l", col="red", main="neg quad, alpha low")
points(kappa_desired, EB_c_endog, t="l", col="blue")
abline(a=0,b=1, lty=3)
detach(df_obj_negative_quadratic_alpha_low)
par(mfrow=c(1,1))
dev.off()


## stack results DFs to summarize c_EB/kappa
df_stacked<-rbind(df_obj_constant, df_obj_increasing, df_obj_negative_quadratic, df_obj_positive_quadratic)
with(df_stacked, summary(FE_c_endog/kappa_desired))
with(df_stacked, summary(EB_c_endog/kappa_desired))

## compare reading info content EB vs FE
set.seed(328)
n_sim_signal_noise<-30000
sim_theta_vec<-rnorm(n_sim_signal_noise, mean=mu_theta, sd=sigma_theta)
sim_n_vec<-n(sim_theta_vec)
sim_eps_mean_vec<-rnorm(n_sim_signal_noise,mean=0,sd=sigma_eps)/sqrt(sim_n_vec)
sim_theta_shocked_vec<-sim_theta_vec+sim_eps_mean_vec
sim_EB_weights<-Vectorize(lambda("EB"))(sim_n_vec)
sim_EB_vec<-sim_EB_weights*sim_theta_shocked_vec
sim_theta_vec_stan<-sim_theta_vec / sd(sim_theta_vec)
sim_theta_shocked_vec_stan<-sim_theta_shocked_vec / sd(sim_theta_shocked_vec)
sim_EB_vec_stan<-sim_EB_vec / sd(sim_EB_vec)

reg_FE<-lm(sim_theta_vec_stan ~ sim_theta_shocked_vec_stan)
reg_EB<-lm(sim_theta_vec_stan ~ sim_EB_vec_stan)

stargazer(reg_FE,reg_EB, type="text")
stargazer(reg_FE,reg_EB)

#r squared ratio is same as signal-to-noise ratio
FE_over_EB_sig_vs_noise<-summary(reg_FE)$sigma^2/summary(reg_EB)$sigma^2

## calibrate hidden action model (Section 5.4)
source("model_HA.R")
###### end compare info content reading

df_obj_LAUSD_reading<-calc_v_vec(q_desired_vec)(0.5)(" ")

pdf("./pdfs/df_obj_LAUSD_reading.pdf")
par(mgp=par_mgp)
plot(q_desired_vec, df_obj_LAUSD_reading$FE_over_EB, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab=expression("Ratio of administrator value" ~ v(kappa)^{"FE"} / v(kappa)^{"EB"}), cex.lab=cex_lab_size) 
abline(h=1, lty=3)
dev.off()

n_LAUSD_teachers<-30000
df_obj_LAUSD_reading$FE_mistakes<-with(df_obj_LAUSD_reading, (1-FE_obj)*n_LAUSD_teachers*2)
df_obj_LAUSD_reading$EB_mistakes<-with(df_obj_LAUSD_reading, (1-EB_obj)*n_LAUSD_teachers*2)
df_obj_LAUSD_reading$EB_over_FE_mistakes<-df_obj_LAUSD_reading$EB_mistakes/df_obj_LAUSD_reading$FE_mistakes
df_obj_LAUSD_reading$EB_minus_FE_mistakes<-df_obj_LAUSD_reading$EB_mistakes-df_obj_LAUSD_reading$FE_mistakes

pdf("./pdfs/df_obj_LAUSD_reading_mistakes.pdf")
par(mgp=par_mgp)
plot(q_desired_vec, df_obj_LAUSD_reading$EB_minus_FE_mistakes, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab="Expected mistakes under EB minus expected mistakes under FE", cex.lab=cex_lab_size)
abline(h=0, lty=3)
dev.off()

#### for summaries in intro, results, conclusion
q_desired_vec_1st_99th<-c(.01, .99)
df_1st_99th_reading<-calc_v_vec(q_desired_vec_1st_99th)(.5)("haiyooo")
df_1st_99th_reading$mistake_frac_FE<-(1-df_1st_99th_reading$FE_obj)*2
df_1st_99th_reading$mistake_frac_EB<-(1-df_1st_99th_reading$EB_obj)*2
df_1st_99th_reading$mistakes_FE<-df_1st_99th_reading$mistake_frac_FE*n_LAUSD_teachers
df_1st_99th_reading$mistakes_EB<-df_1st_99th_reading$mistake_frac_EB*n_LAUSD_teachers
df_1st_99th_reading$mistake_frac_FE_over_EB<-df_1st_99th_reading$mistake_frac_FE/df_1st_99th_reading$mistake_frac_EB
df_1st_99th_reading$mistake_frac_EB_over_FE<-df_1st_99th_reading$mistake_frac_EB/df_1st_99th_reading$mistake_frac_FE
df_1st_99th_reading$mistakes_FE_minus_EB<-df_1st_99th_reading$mistakes_FE-df_1st_99th_reading$mistakes_EB
print(xtable(df_1st_99th_reading, digits=4), file="pdfs/df_mistakes.tex")
write.csv(df_1st_99th_reading, file="pdfs/df_mistakes.csv")
###




### LA Times math n
n_nonvec<-function(theta) max(11, 22.263 -0.225 * theta + -0.039 *theta^2) # 11 is smallest class size in RAND study
n<-function(theta) Vectorize(n_nonvec)(theta)

df_obj_LAUSD_math<-calc_v_vec(q_desired_vec)(0.5)(" ")

pdf("./pdfs/df_obj_LAUSD_math.pdf")
par(mgp=par_mgp)
plot(q_desired_vec, df_obj_LAUSD_math$FE_over_EB, t="l", xlab="Desired cut percentile", ylab=expression(v(kappa)^{"FE"} / v(kappa)^{"EB"}), cex.lab=cex_lab_size, ylim=c(0.997, 1.012))
abline(h=1, lty=3)
dev.off()

df_obj_LAUSD_math$FE_mistakes<-with(df_obj_LAUSD_math, (1-FE_obj)*n_LAUSD_teachers*2 )
df_obj_LAUSD_math$EB_mistakes<-with(df_obj_LAUSD_math, (1-EB_obj)*n_LAUSD_teachers*2 )
df_obj_LAUSD_math$EB_over_FE_mistakes<-df_obj_LAUSD_math$EB_mistakes/df_obj_LAUSD_math$FE_mistakes
df_obj_LAUSD_math$EB_minus_FE_mistakes<-df_obj_LAUSD_math$EB_mistakes-df_obj_LAUSD_math$FE_mistakes

pdf("./pdfs/df_obj_LAUSD_math_mistakes.pdf")
par(mgp=par_mgp)
plot(q_desired_vec, df_obj_LAUSD_math$EB_minus_FE_mistakes, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab="Expected mistakes under EB minus expected mistakes under FE", cex.lab=cex_lab_size)
abline(h=0, lty=3)
dev.off()


### Put LA Times stuff together
## FIG 4a-b
pdf("./pdfs/df_obj_LAUSD.pdf")
par(mgp=par_mgp)
  plot(q_desired_vec, df_obj_LAUSD_reading$FE_over_EB, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab=expression("Ratio of administrator value" ~ v(kappa)^{"FE"} / v(kappa)^{"EB"}), cex.lab=cex_lab_size, col="black")
  points(q_desired_vec, df_obj_LAUSD_math$FE_over_EB, t="l", lty=2, col="red")
  abline(h=1, lty=3)
  legend(0.65, 1.025,c("Reading", "Math"), lty = c(1,2), col=c("black","red"), bty="n")
dev.off()

tikz("./pdfs/df_obj_LAUSD.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
par(mgp=par_mgp)
  plot(q_desired_vec, df_obj_LAUSD_reading$FE_over_EB, t="l", xlab="Desired cut percentile $F(\\kappa)$", ylab="Ratio of administrator value $v_{CP}^{FE}(\\kappa)/v_{CP}^{EB}(\\kappa)$", cex.lab=cex_lab_size, col="black")  
	tmp_par<-par('usr')
  points(q_desired_vec, df_obj_LAUSD_math$FE_over_EB, t="l", lty=2, col="red")
  abline(h=1, lty=3)
  legend(tmp_par[2]*.7, 1.029,c("Reading", "Math"), lty = c(1,2), col=c("black","red"), bty="n")
dev.off()

pdf("./pdfs/df_obj_LAUSD_mistakes.pdf")
par(mgp=par_mgp)
plot(q_desired_vec, df_obj_LAUSD_reading$EB_minus_FE_mistakes, t="l", xlab=expression("Desired cut percentile" ~ F(kappa)), ylab="Expected mistakes under EB minus expected mistakes under FE", col="black", cex.lab=cex_lab_size) 
points(q_desired_vec, df_obj_LAUSD_math$EB_minus_FE_mistakes, t="l", lty=2, col="red")
abline(h=0, lty=3)
legend(0.65, 1400,c("Reading", "Math"), lty = c(1,2), col=c("black","red"), bty="n")
dev.off()

tikz("./pdfs/df_obj_LAUSD_mistakes.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
par(mgp=par_mgp)
  plot(q_desired_vec, df_obj_LAUSD_reading$EB_minus_FE_mistakes, t="l", xlab="Desired cut percentile $F(\\kappa)$", ylab="Difference in expected mistakes (EB - FE)", cex.lab=cex_lab_size, col="black")
	tmp_par<-par('usr')
	points(q_desired_vec, df_obj_LAUSD_math$EB_minus_FE_mistakes, t="l", lty=2, col="red")
  abline(h=1, lty=3)
  legend(tmp_par[2]*.7, tmp_par[4],c("Reading", "Math"), lty = c(1,2), col=c("black","red"), bty="n")
dev.off()


## Hidden action model
source("model_AS0.R")
